import java.io.File;
import java.io.IOException;
import javax.sound.sampled.*;

import java.io.ByteArrayOutputStream;
import java.nio.charset.Charset;

public class Audio extends Thread {
    public static int SAMPLING_RATE = 48000;
    public static int SOUND_FREQ = 880;
    private int times;
    private int msec;
    private String file;
    private String mixerName;
    private int vol = 50;
    private AudioInputStream audio;
    
    public Audio(String mixerName, int vol, int times, int msec) {
    	 this.mixerName = mixerName;
    	 this.vol = vol;
        this.times = times;
        this.msec  = msec;
        audio = new WaveBeep(SAMPLING_RATE, SOUND_FREQ, msec/2);
    }
    
    public Audio(String mixerName, int vol, String file) {
    	 this.mixerName = mixerName;
        this.vol = vol;
        this.file = file;
       
       try {
    	   audio = AudioSystem.getAudioInputStream(new File(file));
       } catch (UnsupportedAudioFileException e) {
    	   e.printStackTrace();
       } catch (IOException e) {
    	   e.printStackTrace();
       }
    }
    
    public Mixer.Info getMixerByName(String name) {
        Mixer.Info[] info = AudioSystem.getMixerInfo();
    	for (int i=0; i<info.length; i++) {
    		if (deMessyCode(info[i].getName()).equals(name)) return info[i];
    	}
    	return null;
    }

    public static String[] getMixerNames() {
        Mixer.Info[] info = AudioSystem.getMixerInfo();
    	String[] ret = new String[info.length];
    	for (int i=0; i<info.length; i++) {
    		ret[i] = deMessyCode(info[i].getName());
    	}
    	return ret;
    }
    
    @Override
    public void run() {
    	Mixer.Info mixerInfo = getMixerByName(mixerName);
    	if (mixerInfo == null) {
    		System.out.println("No device for " + mixerName);
    		return;
    	}
    	//System.out.println("Sound Start");
        try {
            Clip clip = AudioSystem.getClip(mixerInfo);
            clip.open(audio);
            FloatControl control = (FloatControl) clip.getControl(FloatControl.Type.MASTER_GAIN);
            control.setValue((float)Math.log10(vol/100.0f) * 20); // dB
            if (times > 0) {
            	System.out.println("sin wave " + msec + "(ms) * " + times);
            	for (int i=0; i<times; i++) {
            		clip.setFramePosition(0);
            		clip.start();
            		sleep(msec);
            	}
            } else {
            	System.out.println("Sound file \"" + file + "\"");
        		clip.start();
        		sleep(clip.getMicrosecondLength()/1000);
            }
            clip.close();
        } catch (LineUnavailableException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (NegativeArraySizeException e) {
        	//e.printStackTrace(); // PulseAudioClip.java:450
        }
        //System.out.println("Sound End");
    }
    
    /*
     * JDKのバグ対策（サウンドデバイス名のエンコード間違いを直す）
     * https://qiita.com/TakaW/items/023b06cd1ba18ca62047
     * https://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8177951 
     */
    private static final byte HIGH_BIT = (byte) 0b11000000;
    private static String deMessyCode(String messyCode) {
        ByteArrayOutputStream buf = new ByteArrayOutputStream(messyCode.length());
        byte[] originalBytes = messyCode.getBytes(Charset.forName("UTF-8"));
        String systemEnc = System.getProperty("file.encoding");
        //System.out.println(systemEnc + (systemEnc.equalsIgnoreCase("MS932")));

        for (int i = 0; i < originalBytes.length; i++) {
            if ((byte) (originalBytes[i] & HIGH_BIT) == HIGH_BIT) {
            	if (i==originalBytes.length-2 && systemEnc.equalsIgnoreCase("MS932")) break;
            	// SJIS の泣き別れが起きないよう、最後の1文字は書き込まない
                buf.write((originalBytes[i] << 6) | (originalBytes[i + 1] << 2 >>> 2));
                // DELETE 0b110000 and move 6, then delete the 0b10 prefix of the second byte.
                i++;
            } else {
                buf.write(originalBytes[i]);
            }
        }
        return new String(buf.toByteArray(), Charset.forName(systemEnc));
    }
}
