import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;


public class Net extends Thread {
    private Preference preferenc;
    private int device = 0;
    
    public Net(Preference preferenc) {
        this.preferenc = preferenc;
        if (preferenc.watchUSB()) device |= AlertUSB.DEV_USB;
        if (preferenc.watchDVD()) device |= AlertUSB.DEV_DVD;
    }

    public String getURL() {
        String proto = preferenc.noticeNetProto();
        String addr  = preferenc.noticeNetAddr();
        return extractEnv(proto + addr);
    }
    
    public void setParam(int device) {
        this.device = device;
    }

    public String extractEnv(String s) {
        String dev = ":", drv = ":";
        if ((device & AlertUSB.DEV_USB) != 0) {
            dev += "USB:";
            drv += new DeviceList().getAllDevices(AlertUSB.DEV_USB);
        }
        if ((device & AlertUSB.DEV_DVD) != 0) {
            dev += "DVD:";
            drv += new DeviceList().getAllDevices(AlertUSB.DEV_DVD);
        }
        drv = drv.replaceAll("\\\\", "");
        System.out.println("dev = " + dev);
        System.out.println("drv = " + drv);
        try {
            ProcessBuilder pb;
            if (preferenc.isUnix()) {
                pb = new ProcessBuilder(new String[]{ "sh", "-c", "echo \"" + s + "\"" });
            } else {
                pb = new ProcessBuilder(new String[]{ "cmd", "/c", "echo " + s});
            }
            Map<String, String> env = pb.environment(); //環境変数を取得
            env.put("FIND_DEV", dev); //独自変数を追加
            env.put("FIND_DRV", drv); //独自変数を追加
            Process process = pb.start();
            int ret = process.waitFor();
            InputStream is = process.getInputStream();  //標準出力
            BufferedReader br = new BufferedReader(new InputStreamReader(is));
            String t = br.readLine();
            br.close();
            if (t != null) s = t; 
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return s;
    }

    private String result;
    public String getResult() {
        return result;
    }

    @Override
    public void run() {
        URL url;
        try {
            url = new URL(getURL());
        } catch (MalformedURLException e) {
            e.printStackTrace();
            result = "MalformedURL";
            return;
        }
        result = "";
        try {
            // input
            BufferedReader in = new BufferedReader(new
                    InputStreamReader(url.openStream()));
            String s;
            while ( (s=in.readLine()) != null ) {
                if (result.length() > 0) result += ", ";
                result += s;
                System.err.println(s);
            }
            in.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
